#!/usr/bin/perl -w

# LSB Test Execution Framework
# AJAX Server Module (ajax_srv.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my $error = "";
my $data = "";

my $output_xml = <<DATA;
HTTP/1.0 200 OK
Content-type: text/xml

<?xml version="1.0" encoding="UTF-8"?>
DATA

sub list_dir() {
	if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
		$data .= "<profiles>\n";
		foreach (sort(grep(!/^[~\.]/, readdir(DIR)))) {
			$data .= "<name>$_</name>\n";
		}
		$data .= "</profiles>\n";
		closedir(DIR);
	}
	else {
		$error = "Could not obtain directory list!<br />($!)";
	}
}

if (!$_GET{"action"}) {
	$error = "Incorrect call parameters!";
}
elsif ($_GET{"action"} eq "delete") {
	if ($_GET{"profile"}) {
		if (!-f $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"}) {
			$error = "File not found:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		}
		else {
			unlink($SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"});
			list_dir();
		}
	}
	else {
		$error = "No profile name specified!";
	}
}
elsif ($_GET{"action"} eq "save") {
	if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
		system("mkdir -p ".$SERVER_PARAM{"APP_DATA"}."/profiles");
		if (!-d $SERVER_PARAM{"APP_DATA"}."/profiles") {
			$error = "Could not create profile directory:<br />".$SERVER_PARAM{"APP_DATA"}."/profiles";
		}
	}
	if (!$error) {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/<>\|\?\*]/) {
			$error = "Invalid profile name: ".$_POST{"edit_profile_name"};
		}
	}
	if (!$error) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_POST{"edit_profile_name"};
		if (open(FILE, ">$full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_platform(`uname -m`));
			my $lsb_version = ($_POST{"lsb_version"} or "3.1");
			my $lsb_profile = ($_POST{"lsb_profile"} or "desktop");
			print FILE "CERTIFY:         ".$_POST{"certify"}."\n"      if ($_POST{"certify"});
			print FILE "NAME:            ".$_POST{"tester_name"}."\n"  if ($_POST{"tester_name"});
			print FILE "ORGANIZATION:    ".$_POST{"organization"}."\n" if ($_POST{"organization"});
			if ($_POST{"tester_email"}) {
				print FILE "EMAIL:           ".$_POST{"tester_email"}."\n";
				print FILE "SEND_EMAIL:      ".($_POST{"send_email_report"} ? "1" : "0")."\n";
			}

			print FILE "VERBOSE_LEVEL:   ".($_POST{"verbose_level"} ? $_POST{"verbose_level"} : "1")."\n";
			print FILE "KEEP_PACKAGES:   ".($_POST{"keep_packages"} ? "1" : "0")."\n";
			print FILE "SHORT_SCENARIOS: ".($_POST{"short_scen"} ? "1" : "0")."\n";
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "USE_INTERNET:    ".($_POST{"use_internet"} ? "1" : "0")."\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			my $manual_present = 0;
			my $automatic_present = 0;
			my $test_suites = '';
			foreach (sort grep /mod-$architecture-$lsb_version-/, keys %_POST) {
				my $tmp = $_;
				$tmp =~ s/^mod-/modstage-/;
				s/^mod-$architecture-$lsb_version-//;
				$test_suites .= "$_ ".$_POST{$tmp}."\n";
				if ($_ eq 'manappbat') {
					$manual_present = 1;
				}
				else {
					$automatic_present = 1;
				}
				$tmp =~ s/^modstage-/modopt-/;
				if ($_POST{$tmp}) {
					$test_suites .= "$_-opt ".$_POST{$tmp}."\n";
				}
			}
			print FILE "#MANUAL_TESTS:    $manual_present\n";
			print FILE "#AUTOMATIC_TESTS: $automatic_present\n";
			print FILE $test_suites;

			close(FILE);
			list_dir();
		}
		else {
			$error = "Cannot open file for writing ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{"action"} eq "load") {
	my %profile_data = (
		"tester_name" => "",
		"organization" => "",
		"tester_email" => "",
		"send_email_report" => "0",
		"architecture" => guess_platform(`uname -m`),
		"use_internet" => "1",
		"keep_packages" => "1",
		"short_scen" => "0",
		"lsb_version" => "3.1",
		"lsb_profile" => "desktop",
		"verbose_level" => "1",
		"comments" => ""
	);
	my %tests_options = ("cmdchk" => 3, "libchk" => 3, "desktop" => 3);
	if (!$_GET{"profile"}) {
		$error = "No profile name specified!";
	}
	if (!$error) {
		my $full_save_name = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_GET{"profile"};
		if (open(FILE, $full_save_name)) {
			my @comments = ();
			while (<FILE>) {
				s/[\r\n]//g;
				if (/^CERTIFY:\s+(.*)$/) {
				}
				elsif (/^NAME:\s+(.*)$/) {
					$profile_data{"tester_name"} = $1;
				}
				elsif (/^ORGANIZATION:\s+(.*)$/) {
					$profile_data{"organization"} = $1;
				}
				elsif (/^EMAIL:\s+(.*)$/) {
					$profile_data{"tester_email"} = $1;
				}
				elsif (/^SEND_EMAIL:\s+(.*)$/) {
					$profile_data{"send_email_report"} = $1;
				}
				elsif (/^VERBOSE_LEVEL:\s+(.*)$/) {
					$profile_data{"verbose_level"} = $1;
				}
				elsif (/^KEEP_PACKAGES:\s+(.*)$/) {
					$profile_data{"keep_packages"} = $1;
				}
				elsif (/^SHORT_SCENARIOS:\s+(.*)$/) {
					$profile_data{"short_scen"} = $1;
				}
				elsif (/^LSB_VERSION:\s+(.*)$/) {
					$profile_data{"lsb_version"} = $1;
				}
				elsif (/^LSB_PROFILE:\s+(.*)$/) {
					$profile_data{"lsb_profile"} = $1;
				}
				elsif (/^ARCHITECTURE:\s+(.*)$/) {
					$profile_data{"architecture"} = $1;
				}
				elsif (/^USE_INTERNET:\s+(.*)$/) {
					$profile_data{"use_internet"} = $1;
				}
				elsif (/^COMMENTS: (.*)$/) {
					push @comments, $1;
				}
				elsif (/^[^\s#]/) {
					my ($mod, $stage) = split(/\s+/, $_, 2);
					if ($mod =~ m/^(.*)-opt$/) {
						$tests_options{$1} = $stage;
					}
					else {
						$profile_data{"modules"}->{$mod} = $stage;
					}
				}
			}
			close(FILE);
			$profile_data{"comments"} = join("\n", @comments);
			if ($profile_data{"tester_email"} eq "") {
				$profile_data{"send_email_report"} = "";
			}
			$data .= "<profile_data>\n";
			foreach my $name (qw/architecture lsb_version tester_name organization tester_email lsb_profile verbose_level comments/) {
				$data .= "<item type=\"value\"><name>$name</name><value>$profile_data{$name}</value></item>\n" if (defined($profile_data{$name}));
			}
			foreach my $name (qw/send_email_report use_internet keep_packages short_scen/) {
				$data .= "<item type=\"check\"><name>$name</name><value>".($profile_data{$name} ? "1" : "0")."</value></item>\n" if (defined($profile_data{$name}));
			}
			foreach my $mod (keys %{$profile_data{"modules"}}) {
				$data .= "<item type=\"mod\"><name>$mod</name><value>".$profile_data{"modules"}->{$mod}."</value></item>\n";
			}
			foreach my $mod (keys %tests_options) {
				$data .= "<item type=\"modopt\"><name>$mod</name><value>$tests_options{$mod}</value></item>\n";
			}
			$data .= "</profile_data>\n";
			list_dir();
		}
		else {
			$error = "Cannot open file for reading ($!):<br />$full_save_name";
		}
	}
}
else {
	$error = "Incorrect call parameters!";
}

if ($error) {
	$error = "<ul><li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$error</font></li></ul>";
	$error =~ s/&/&amp;/g;
	$error =~ s/</&lt;/g;
	$error =~ s/>/&gt;/g;
	print $output_xml."<root>\n<error>".$error."</error>\n</root>";
}
else {
	print $output_xml."<root>\n".$data."</root>";
}
